#include <stdio.h>
#include <windows.h>

int wmain(int argc, wchar_t** argv)
{
    if (argc < 3)
    {
        printf("Usage: HostName ServiceName\n");
        return 1;
    }

    auto scm = OpenSCManager(argv[1], nullptr, MAXIMUM_ALLOWED);
    if (scm == nullptr)
    {
        printf("[ERROR]: Failed to open SCM error:%d\n", GetLastError());
        return 0;
    }

    auto service = OpenService(scm, argv[2], SERVICE_START);
    if (service)
    {
        printf("[ERROR]: Was granted SERVICE_START when we didn't expect it. Maybe using the wrong user?\n");
        return 0;
    }

    DWORD error = GetLastError();
    if (error == ERROR_SERVICE_DOES_NOT_EXIST)
    {
        printf("[ERROR]: Service doesn't exist.\n");
        return 0;
    }

    if (error != ERROR_ACCESS_DENIED)
    {
        printf("[ERROR]: Expected ERROR_ACCESS_DENIED but got %d, the rest might not work.\n", GetLastError());
    }

    service = OpenService(scm, argv[2], MAXIMUM_ALLOWED);
    if (service == nullptr)
    {
        printf("[ERROR]: Failed to open service error:%d\n", GetLastError());
        return 0;
    }

    error = ERROR_SUCCESS;
    if (!StartService(service, 0, nullptr))
    {
        error = GetLastError();
    }

    switch (error)
    {
        case ERROR_ACCESS_DENIED:
            printf("[ERROR]: Got access denied error.\n");
            break;
        case ERROR_SUCCESS:
            printf("[SUCCESS]: Successfully started service.\n");
            break;
        case ERROR_SERVICE_DISABLED:
            printf("[SUCCESS]: Service was disabled but passed the access check.\n");
            break;
        default:
            printf("[WARNING]: Didn't start service but didn't get back expected error:%d\n", error);
    }

    return 0;
}
